<?php

/**
 * @file all functions needed to make contacts accessible by services
 * @author Manuel Pistner <pistner@brightsolutions.de>
 * @copyright (c) 2013
 */

/**
 * Create a new contact node by services module request
 * @param array $contact the data delivered by the webservice, $contact['contact'] must be an array
 * @return the contact node that was created or an error of contact could not be created
 * @TODO: maybe use drupal_form_submit or _node_resource_create to have validation instead of node_save
 */
function _erpal_basic_service_contact_create($contact) {
  $returns = array();
  global $user;

  $value = $contact['contact'];

  //if we don't have title, take the first 65 chars from body
  $firstname = isset($value['firstname']) ? $value['firstname'] : '';
  $lastname = isset($value['lastname']) ? $value['lastname'] : '';
  $company = isset($value['company']) ? $value['company'] : '';
  $email = isset($value['email']) ? $value['email'] : '';
  $origin = isset($value['origin']) ? $value['origin'] : false;
  $nid = isset($value['nid']) ? $value['nid'] : false;
  $phone = isset($value['phone']) ? $value['phone'] : false;
  $vat_number = isset($value['vat_number']) ? $value['vat_number'] : false;
  $criteria = isset($value['criteria']) ? $value['criteria'] : array();

  $addresses = isset($value['addresses']) ? $value['addresses'] : array();

  //check if there is already a contact with the given name and emailadress, if so, return this nid 
  $contact_nid = false;
  if (count($criteria))
    $contact_nid = erpal_lib_helper_entity_check_exits($criteria, 'node', 'erpal_contact');

  //create the contact node obect, new one if none exists or load the node
  if (!$contact_nid) {
    $contact_node_arr = array('type' => 'erpal_contact');
    $contact_node = (object) $contact_node_arr;
  }
  else {
    $contact_node = node_load($contact_nid);
  }

  $form_id = 'erpal_contact_node_form';
  $form_values = array(
    'language' => LANGUAGE_NONE,
    'name' => $user->name,
    'status' => 1,
    'type' => 'erpal_contact',
    'promote' => 0,
    'sticky' => 0,
  );

  //now add all the fields if they are set
  if ($firstname)
    $form_values['field_firstname'] = array(LANGUAGE_NONE => array(0 => array('value' => $firstname)));
  if ($lastname)
    $form_values['field_lastname'] = array(LANGUAGE_NONE => array(0 => array('value' => $lastname)));
  if ($company)
    $form_values['field_company_name'] = array(LANGUAGE_NONE => array(0 => array('value' => $lastname)));
  if ($email)
    $form_values['field_email'] = array(LANGUAGE_NONE => array(0 => array('value' => $email)));
  if ($origin) {
    $tid = _erpal_basic_service_check_and_create_term($origin, 'activity_origin');
    $form_values['field_activity_origin'][LANGUAGE_NONE]['value'] = $tid;
  }

  if (false && $contact_tag) {
    //@TODO: add contact tag
    $form_values['field_contact_tags'] = array(LANGUAGE_NONE => array(0 => array('tid' => $contact_tag)));
  }
  //@TODO: handle relations may be company and a person is set, we need to save both
  //@TODO: TEST FROM HERE TEST TEST TEST FROM HERE
  //@TODO: handle field collections and taxonomy terms, create new if not exist 
//  $addresses = array();
  $phones = array();
  if ($phone) {
    //@TODO: with setting delta=0 the current number with delta = 0 gets lost. So we need to find the correct delta if there are already values set
    $phones[LANGUAGE_NONE][0]['field_phone_number'] = array(LANGUAGE_NONE => array(0 => array('value' => $phone)));
  }

  //add only new addresses
  $form_values['field_addresses'] = _erpal_basic_service_check_and_add_addresses($addresses, $contact_node);

  if ($vat_number)
    $form_values['field_vat_number'] = array(LANGUAGE_NONE => array(0 => array('value' => $vat_number)));

  $form_values['field_phone'] = $phones;



  // if there is no contact, create one otherwise it will be updated
  // node module stores form processing functions in a
  // seperate include entitled node.pages.inc
  // so must load that before executing form
  module_load_include('inc', 'node', 'node.pages');
  $form_values['op'] = t('Save');  // this seems to be a required value
  $form_state['values'] = $form_values;
  //we need to set items_count, if we add multiple field entries 

  $addresses_count = count($contact_node->field_addresses[LANGUAGE_NONE]);
  $new_addresses_count = count($form_values['field_addresses'][LANGUAGE_NONE]);
  $form_state['field']['field_addresses'][LANGUAGE_NONE]['items_count'] = $addresses_count + $new_addresses_count;
  drupal_form_submit($form_id, $form_state, (object) $contact_node);

  $contact_node = $form_state['node'];

  $errors = form_get_errors();

  return _erpal_basic_service_create_service_response($errors, $contact_node);
}

/**
 * callback for retrieving a single contact resource.
 *
 * @param int $id the nid of the project node that should be retrieved
 * @param bool $inlude_children, instead of url, include short version of children, defaults to false
 * @return object
 */
function _erpal_basic_service_contact_retrieve($id, $include_children = FALSE) {
  $contact_node = _erpal_basic_service_contact_out((int) $id, TRUE, (bool) $include_children);
  return $contact_node;
}

/**
 * Callback for returning a list of contacts respecting given filters
 * @param int $start number to start list from, defaults to 0
 * @param int $count number of item to return, defaults to PHP_INT_MAX
 * @param int $changed_since UTC timestamp, deliver only tasks changed after that timestamp , defaults to 0
 * @param bool $details whether tasks will be returned with details , defaults to false
 * @return array array of tasks, prepared for output
 */
function _erpal_basic_service_contact_index($start = 0, $count = PHP_INT_MAX, $changed_since = 0, $details = false) {

  $query = new EntityFieldQuery();
  $query->entityCondition('entity_type', 'node')
      ->propertyCondition('type', 'erpal_contact')
      ->range((int) $start, (int) $count)
      ->propertyCondition('changed', (int) $changed_since, '>=');

  $entities = $query->execute();

  $contacts_out = array();

  if (!empty($entities['node'])) {
    foreach (array_keys($entities['node']) as $nid) {
      $contacts_out[] = _erpal_basic_service_contact_out($nid, $details);
    }
  }

  return $contacts_out;
}

/**
 * retrieves a contact node and returns only simple data
 * @param int $nid the node id of the project to return
 * @param bool $details whether or not to return the project with details
 * @return array the project prepared for output
 */
function _erpal_basic_service_contact_out($nid, $details = TRUE) {

  // load contact node
  $contact = node_load($nid);
  //dd($contact, 'contact_out');
  // if given node id is not a contact, return error
  if ($contact->type !== 'erpal_contact') {
    return services_error('failed loading node. nid ' . $nid . ' is not a contact', 400);
  }

  // load wrapper
  $contact_wrapper = entity_metadata_wrapper('node', $contact);

  // construct response
  $contact_out = array();

  // base data
  $contact_out['created'] = (int) $contact->created;
  $contact_out['changed'] = (int) $contact->changed;
  $contact_out['url'] = url('rest/basic/contact/' . $contact->nid, array('absolute' => TRUE));

  // revision
  $contact_out['revision_timestamp'] = (int) $contact->revision_timestamp;
  $contact_out['revision_id'] = (int) $contact->revision_uid;

  // contact type
  $contact_type = $contact_wrapper->field_contact_type->value();
  $contact_out['type'] = $contact_type;

  // name
  if ($contact_type == 'person') {
    $contact_out['first_name'] = $contact_wrapper->field_firstname->value();
    $contact_out['name'] = $contact_wrapper->field_lastname->value();
    $contact_out['gender'] = $contact_wrapper->field_gender->value();
  }
  else if ($contact_type == 'company') {
    $contact_out['company'] = $contact_wrapper->field_company_name->value();
  }

  if ($details) {

    // addresses
    $contact_out['addresses'] = array();
    foreach ($contact_wrapper->field_addresses->getIterator() as $key => $address) {
      $a_name = $address->field_address_name->value();
      $a_city = $address->field_city->value();
      $a_street = $address->field_street->value();
      $a_zip_code = $address->field_zip_code->value();
      $a_country = $address->field_country_term->name->value();
      array_push($contact_out['addresses'], array(
        'name' => $a_name ? $a_name : '',
        'city' => $a_city ? $a_city : '',
        'street' => $a_street ? $a_street : '',
        'zip_code' => $a_zip_code ? $a_zip_code : '',
        'country' => $a_country ? $a_country : '',
      ));
    }
    // image
    if (!empty($contact->field_contact_image)) {
      $image_array = $contact_wrapper->field_contact_image->value();
      $contact_out['image'] = url('sites/default/files/' . $image_array['filename'], array('absolute' => TRUE));
    }
    // phones
    $contact_out['phones'] = array();
    foreach ($contact_wrapper->field_phone->getIterator() as $key => $phone) {
      array_push($contact_out['phones'], array(
        'type' => '', // $phone->field_number_type_terms->name->value(), // FIXME
        'number' => $phone->field_phone_number->value(),
      ));
    }
    // faxes
    if (!empty($contact->field_fax)) {
      $contact_out['fax'] = $contact_wrapper->field_fax->value();
    }
    // emails
    if (!empty($contact->field_email)) {
      $contact_out['mail'] = $contact_wrapper->field_email->value();
    }
    // websites
    $contact_out['websites'] = array();
    foreach ($contact_wrapper->field_website->getIterator() as $key => $website) {
      array_push($contact_out['websites'], $website->url->value());
    }
    // relations - TODO
    $contact_out['relations'] = array();
    // files
    $contact_out['files'] = array();
    foreach ($contact_wrapper->field_asset_files->getIterator() as $key => $file) {
      $file_array = $file->field_docs_file->value();
      if (!empty($file_array)) {
        array_push($contact_out['files'], array(
          'filename' => $file_array['filename'],
          'mimetype' => $file_array['filemime'],
          'filesize' => (int) $file_array['filesize'],
          'url' => file_create_url($file_array['uri']),
          'description' => ($file_array['description']) ? $file_array['description'] : '',
          'fid' => (int) $file_array['fid'],
        ));
      }
    }
    // birthday
    if (!empty($contact->field_birthday)) {
      $contact_out['birthday'] = substr($contact->field_birthday[LANGUAGE_NONE][0]['value'], 0, 10); // $contact_wrapper->field_birthday->raw();
    }
    // social
    $contact_out['social'] = array();
    if ($contact->field_xing) {
      $contact_out['social']['xing'] = $contact_wrapper->field_xing->url->value();
    }
    if ($contact->field_facebook) {
      $contact_out['social']['facebook'] = $contact_wrapper->field_facebook->url->value();
    }
    if ($contact->field_twitter) {
      $contact_out['social']['twitter'] = $contact_wrapper->field_twitter->url->value();
    }
    if ($contact->field_google_plus) {
      $contact_out['social']['google'] = $contact_wrapper->field_google_plus->url->value();
    }
    // comments
    $contact_out['comments'] = array();
    $query = new EntityFieldQuery();
    $query->entityCondition('entity_type', 'comment')
        ->propertyCondition('nid', $nid);
    $entities = $query->execute();
    if (!empty($entities['comment'])) {
      foreach (array_keys($entities['comment']) as $cid) {
        $comment = comment_load($cid);
        if ($comment) {
          $comment_body = $comment->comment_body[LANGUAGE_NONE][0];
          array_push($contact_out['comments'], array(
            'created' => (int) $comment->created,
            'changed' => (int) $comment->changed,
            'cid' => (int) $comment->cid,
            'subject' => ($comment->subject) ? $comment->subject : '',
            'name' => ($comment->name) ? $comment->name : '',
            'registered_name' => ($comment->registered_name) ? $comment->registered_name : '',
            'comment_body' => $comment_body['value'],
            'comment_format' => $comment_body['format'],
          ));
        }
      }
    }
  }

  return $contact_out;
}

/**
 * Creates an array as the response of a webservice request
 */
function _erpal_basic_service_create_service_response($errors, $contact_node) {
  if (!empty($errors)) {
    $returns['error'] = TRUE;
    $returns['node'] = false;
    foreach ($errors as $key => $value) {
      $error_string .= $key . ": " . $value . "<br/>";
    }
    $returns['message'] = 'Contact could not be created because of some errors: <br/>' . $error_string;
  }
  else {
    $returns['node'] = (array) $contact_node;
    $returns['error'] = false;
    $returns['message'] = '';
  }

  return $returns;
}

/**
 * Check access for contact node
 * @param $op
 *   The operation to be performed on the node. Possible values are:
 *   - "view"
 *   - "update"
 *   - "delete"
 *   - "create"
 */
function _erpal_basic_service_contact_access($op, $args = NULL) {
  return node_access($op, 'erpal_contact');
}

/**
 * Check if the given addresses exist in the given node, if not return form string
 * all keys must be given
 * @param array $addresses array('address_name'->'','addition_to_address'->'','street'->'','zip'->'','city'->'','address_type','country'->'')
 * @param node object $contact_node
 */
function _erpal_basic_service_check_and_add_addresses($addresses, $contact_node) {
  //loop through given node and add all save addresses(serailized) in the needed format to an array
  $existing_addresses = array();
  if (!empty($contact_node->field_addresses[LANGUAGE_NONE])) {
    foreach ($contact_node->field_addresses[LANGUAGE_NONE] as $key => $address) {
      $test = entity_load('field_collection_item', array($address['value']));
      $existing_address = array();
      $existing_address['address_name'] = (string) $test[$address['value']]->field_address_name[LANGUAGE_NONE][0]['value'];
      $existing_address['addition_to_address'] = (string) $test[$address['value']]->field_addition_to_address[LANGUAGE_NONE][0]['value'];
      $existing_address['street'] = (string) $test[$address['value']]->field_street[LANGUAGE_NONE][0]['value'];
      $existing_address['zip'] = (string) $test[$address['value']]->field_zip_code[LANGUAGE_NONE][0]['value'];
      $existing_address['city'] = (string) $test[$address['value']]->field_city[LANGUAGE_NONE][0]['value'];
      $existing_address['address_type'] = (string) taxonomy_term_load($test[$address['value']]->field_address_type_terms[LANGUAGE_NONE][0]['tid'])->name;
      $existing_address['country'] = (string) taxonomy_term_load($test[$address['value']]->field_country_term[LANGUAGE_NONE][0]['tid'])->name;
      $existing_addresses[] = serialize($existing_address);
    }
  }

  //TODO: normalize $addresses array so missing keys are empty strings and order of keys are the same as from the saved addresses    //loop through given addresses and check if they are in array of node adresses, if not, add to new_addresses array
  $new_addresses = array();
  foreach ($addresses as $address) {
    //normalize the sent address, so its equal in structure to the serialized existing addresses
    $address_to_test = array();
    $address_to_test['address_name'] = (string) $address['address_name'];
    $address_to_test['addition_to_address'] = (string) $address['addition_to_address'];
    $address_to_test['street'] = (string) $address['street'];
    $address_to_test['zip'] = (string) $address['zip'];
    $address_to_test['city'] = (string) $address['city'];
    $address_to_test['address_type'] = (string) $address['address_type'];
    $address_to_test['country'] = (string) $address['country'];
    if (!in_array(serialize($address_to_test), $existing_addresses)) {
      $new_addresses[] = $address_to_test;
    }
  }

  if (!empty($new_addresses)) {
    $counter = 1;
    $form_values = array();
    foreach ($new_addresses as $address) {
      if (isset($address['address_name']))
        $form_values[LANGUAGE_NONE][$last_key + $counter]['field_address_name'] = array(LANGUAGE_NONE => array(0 => array('value' => $address['address_name'])));
      if (isset($address['addition_to_address']))
        $form_values[LANGUAGE_NONE][$last_key + $counter]['field_addition_to_address'] = array(LANGUAGE_NONE => array(0 => array('value' => $address['addition_to_address'])));
      if (isset($address['street']))
        $form_values[LANGUAGE_NONE][$last_key + $counter]['field_street'] = array(LANGUAGE_NONE => array(0 => array('value' => $address['street'])));
      if (isset($address['zip']))
        $form_values[LANGUAGE_NONE][$last_key + $counter]['field_zip_code'] = array(LANGUAGE_NONE => array(0 => array('value' => $address['zip'])));
      if (isset($address['city']))
        $form_values[LANGUAGE_NONE][$last_key + $counter]['field_city'] = array(LANGUAGE_NONE => array(0 => array('value' => $address['city'])));
      if (!empty($address['address_type'])) {
        $tid = _erpal_basic_service_check_and_create_term($address['address_type'], 'address_types');
        $form_values[LANGUAGE_NONE][$last_key + $counter]['field_address_type_terms'] = array(LANGUAGE_NONE => array('tid' => $tid));
      }
      if (!empty($address['country'])) {
        $tid = _erpal_basic_service_check_and_create_term($address['country'], 'countries');
        $form_values[LANGUAGE_NONE][$last_key + $counter]['field_country_term'] = array(LANGUAGE_NONE => array('tid' => $tid));
      }

      $counter++;
    }
  }
  return $form_values;
}

/**
 * 
 * @param type $term_name The name of a term
 * @param type $vocabulary_name the name of a vocabulary
 * @return type the tid of the term that was found and created
 */
function _erpal_basic_service_check_and_create_term($term_name, $vocabulary_name) {
  //check if there is a term for this country, if not, create it
  $term = taxonomy_get_term_by_name($term_name, $vocabulary_name);
  $term = reset($term);

  if ($term && !empty($term->tid)) {
    $tid = $term->tid;
  }
  else {
    //@TODO: make it configurable wether to create a new one or return an error
    $term = new stdClass();
    $term->name = $term_name;
    $vocab = taxonomy_vocabulary_machine_name_load($vocabulary_name);
    $term->vid = $vocab->vid;
    taxonomy_term_save($term);
    $tid = $term->tid;
  }
  return $tid;
}
