<?php

/**
 * @file functions uses for the file ressource
 */
function _erpal_docs_service_file_download($erpal_file_nid) {
  //download the file attached to the current nid
  $file_node = node_load($erpal_file_nid);
  $file = isset($file_node->field_docs_file[LANGUAGE_NONE][0]) ? $file_node->field_docs_file[LANGUAGE_NONE][0] : false;
  if ($file) {

    $fid = $file['fid'];
    $file = file_load($fid);
    $uri = $file->uri;
    $path = file_create_url($uri);
    return array('path' => $path);
  }

  return false;
}

/**
 * Check access for the ressource to download a file (access on a concrete file is checked)
 * with node_access
 */
function _erpal_docs_service_file_download_access($op, $args) {
  $nid = isset($args[0]) ? $args[0] : false;
  if (!$nid)
    return false;
  if (is_numeric($nid))
    $node = node_load($nid);
  else
    $node = $nid;
  return node_access($op, $node);
}

/**
 * Check access for contact node
 * @param $op
 *   The operation to be performed on the node. Possible values are:
 *   - "view"
 *   - "update"
 *   - "delete"
 *   - "create"
 */
function _erpal_docs_service_file_access($op, $args = NULL) {
  return node_access($op, 'erpal_file');
}

/**
 * callback for returning a file
 * 
 * @param int $nid the node id of the file to return
 * @param bool $details whether to returned detailed information, defaults to true
 * @return array the formatted file node for output
 */
function _erpal_docs_service_file_retrieve($nid, $details = TRUE) {
  return _erpal_docs_service_file_out($nid);
}

/**
 * callback for returning a list of files
 * 
 * @param int $start number to start list from, defaults to 0
 * @param int $count number of item to return, defaults to PHP_INT_MAX
 * @param int $changed_since UTC timestamp, deliver only tasks changed after that timestamp , defaults to 0
 * @return array array of files
 */
function _erpal_docs_service_file_index($start = 0, $count = PHP_INT_MAX, $changed_since = 0, $details = FALSE) {
  $query = new EntityFieldQuery();
  $query->entityCondition('entity_type', 'node')
      ->propertyCondition('type', 'erpal_file')
      ->range((int) $start, (int) $count)
      ->propertyCondition('changed', (int) $changed_since, '>=');

  $entities = $query->execute();

  $files_out = array();

  if (!empty($entities['node'])) {
    foreach (array_keys($entities['node']) as $nid) {
      $files_out[] = _erpal_docs_service_file_out($nid, $details);
    }
  }

  return $files_out;
}

/**
 * retrieves a file node and generates corresponding response output
 * @param int $nid the node id of the file to return
 * @param bool $details whether to returned detailed information, defaults to true
 * @return array the formatted file node for output
 */
function _erpal_docs_service_file_out($nid, $details = TRUE) {

  // load contact node
  $file = node_load($nid);
  dd($file, 'file_out');

  // if given node id is not a contact, return error
  if ($file->type !== 'erpal_file') {
    return services_error('failed loading node. nid ' . $nid . ' is not a file', 400);
  }

  // load wrapper
  $file_wrapper = entity_metadata_wrapper('node', $file);

  // base data
  $files_out = array();
  $files_out['title'] = $file->title;
  $files_out['created'] = (int) $file->created;
  $files_out['changed'] = (int) $file->changed;
  $files_out['url'] = url('rest/erpal_files/erpal_file/' . $file->nid, array('absolute' => TRUE));

  // revision
  $files_out['revision_timestamp'] = (int) $file->revision_timestamp;
  $files_out['revision_id'] = (int) $file->revision_uid;

  // file data
  $file_array = $file_wrapper->field_docs_file->value();
  if (!empty($file_array)) {
    $files_out['filename'] = $file_array['filename'];
    $files_out['mimetype'] = $file_array['filemime'];
    $files_out['filesize'] = (int) $file_array['filesize'];
    $files_out['download_url'] = file_create_url($file_array['uri']);
    $files_out['description'] = ($file_array['description']) ? $file_array['description'] : '';
    $files_out['fid'] = (int) $file_array['fid'];
  }

  if ($details) {
    
    // comments
    $files_out['comments'] = array();
    $query = new EntityFieldQuery();
    $query->entityCondition('entity_type', 'comment')
        ->propertyCondition('nid', $nid);
    $entities = $query->execute();
    if (!empty($entities['comment'])) {
      foreach (array_keys($entities['comment']) as $cid) {
        $comment = comment_load($cid);
        if ($comment) {
          $comment_body = $comment->comment_body[LANGUAGE_NONE][0];
          array_push($files_out['comments'], array(
            'created' => (int) $comment->created,
            'changed' => (int) $comment->changed,
            'cid' => (int) $comment->cid,
            'subject' => ($comment->subject) ? $comment->subject : '',
            'name' => ($comment->name) ? $comment->name : '',
            'registered_name' => ($comment->registered_name) ? $comment->registered_name : '',
            'comment_body' => $comment_body['value'],
            'comment_format' => $comment_body['format'],
          ));
        }
      }
    }
    
  }
  
  return $files_out;
}
