<?php

/**
 * Plugin to export as docx
 */
$plugin = array(
  'type' => 'Text-Document',
  'handler' => array(
    'class' => 'document_ooxml_export',
  ),
);
  
class document_ooxml_export extends document_export {
  
  function __construct($options){
    parent::__construct($options);
    if(document_ooxml_export_require_htmltodocx()){ 
      $this->document = $this->create_document($options);  
    } else {
      return false;
    }
  }

  public function save($uri, $filename){

    global $user;
    
    $filepath = drupal_realpath($uri.$filename);
    $filepath = file_destination($filepath, FILE_EXISTS_RENAME);
    
    $obj_writer = PHPWord_IOFactory::createWriter($this->document, 'Word2007');
    $obj_writer->save($filepath);
    
    
    // Begin building file object.
    $file = new stdClass();
    $file->uid      = $user->uid;
    $file->status   = 1; //final!
    $file->filename = $filename;
    $file->uri      = $uri.basename($filepath);
    $file->filemime = file_get_mimetype($filepath);
    $file->filesize = filesize($filepath);   
    file_save($file);
    return $file;    
  }
  
  public function stream($filename){
    $temp_file = tempnam(sys_get_temp_dir(), 'PHPWord');
    $obj_writer = PHPWord_IOFactory::createWriter($this->document, 'Word2007');
    $obj_writer->save($temp_file);
    $this->stream_file($filename, $temp_file);
  }
  
  private function create_document($options){
  
    //Create a new OOXML Document and add a new Section  
    $phpword_object = new PHPWord();
    $section = $phpword_object->createSection();
    
    //Create configuration array for htmltodocx    
    $domain = (isset($GLOBALS['base_url']))
        ? $GLOBALS['base_url']
        : '';  
    // Settings 
    $settings = array(
      // Required parameters:
      'phpword_object' => &$phpword_object,
      'base_root' => $domain,
      'base_path' => '',
      
      // Optional parameters - showing the defaults if you don't set anything:
      'current_style' => array('size' => '11'), 
      'parents' => array(0 => 'body'),
      'list_depth' => 0,
      'context' => 'section',
      'pseudo_list' => TRUE, 
      'pseudo_list_indicator_font_name' => 'Calibri',
      'pseudo_list_indicator_font_size' => '11',
      'pseudo_list_indicator_character' => ' • ',
      'table_allowed' => TRUE,        
      'style_sheet' => document_ooxml_export_get_style(),
    );
   
    if(isset($options['toc']) && ($options['toc'] == TRUE)){
      $settings += array(
        'structure_document' => TRUE, 
        'table_of_contents_id' => erpal_document_export_get_toc_id(),
      );
      $toc = '<h1>Table of contents:</h1><div id="'.erpal_document_export_get_toc_id().'"></div>';
      
      $html_dom = $this->prepare_page($toc);
      $html_dom_array = $html_dom->find('html', 0)->children();
      htmltodocx_insert_html($section, $html_dom_array[0]->nodes, $settings);
      $section->addPageBreak();
      $html_dom->clear();
      unset($html_dom); 
    } 
    
    // Add frontpage if set
    if(isset($options['frontpage']) && $options['frontpage']){
      $html_dom = $this->prepare_page($options['frontpage']);
      $str = $html_dom;
      $html_dom_array = $html_dom->find('html', 0)->children();
      htmltodocx_insert_html($section, $html_dom_array[0]->nodes, $settings);
      // $section->addPageBreak();
      $html_dom->clear();
      unset($html_dom); 
      $section->addPageBreak();
    }
       
    // Set Footer if set
    if(isset($this->footer)){
      $new_settings = $settings;
      
      $new_settings['style_sheet']['elements']['table'] = array (
        'border' => 'none',
      );
      $new_settings['style_sheet']['elements']['th'] = array (
        'border' => 'none',
      );
      $new_settings['style_sheet']['elements']['td'] = array (
        'border' => 'none',
      );
      
      $new_settings['style_sheet']['default'] = array (
        'size' => 7,
        'name' => 'Calibri',
      );
      
      $new_settings['current_style'] = array('size' => '7');
      
      $footer = $section->createFooter();
      $html_dom = $this->prepare_page($this->footer);
      $html_dom_array = $html_dom->find('html', 0)->children();
      htmltodocx_insert_html($footer, $html_dom_array[0]->nodes, $new_settings);
      $html_dom->clear();
      unset($html_dom);
    } 
    
    // Set Header if set
    if(isset($this->header)){
      $new_settings = $settings; 
     
      $header = $section->createHeader();
      
      // If information is given about the header size resize it to fit exactly 
      // on the page 
      // Dirty but it works
      if(isset($this->header['size'])) {
        
        $size = $this->header['size'];
      
        if($size['width'] > 550){
          $size['height'] = intval($size['height'] * 550 / $size['width']);
          $size['width'] = 550;
          $new_settings['style_sheet']['classes']['auto-scale'] = $size;
        }
      }
      $html_dom = $this->prepare_page($this->header['html']);
      $html_dom_array = $html_dom->find('html', 0)->children();
      htmltodocx_insert_html($header, $html_dom_array[0]->nodes, $new_settings);
      $html_dom->clear();
      unset($html_dom);
    }
    
    // Add pages 
    foreach($this->pages as $page){
      // Parse our html with simple html dom
      $html_dom = $this->prepare_page($page);
      $html_dom_array = $html_dom->find('html', 0)->children();
      htmltodocx_insert_html($section, $html_dom_array[0]->nodes, $settings);
      
      if(isset($options['page_breaks']) && ($options['page_breaks'] == TRUE))
        $section->addPageBreak();
      
      $html_dom->clear();
      unset($html_dom);  
      
    }  
    return $phpword_object;   
  }
  
  private function prepare_page($data){
    
    $html = 
      '<html><body>'
      .$this->prepare_images($data)
      .'</body></html>';
      
      
    $html_dom = new simple_html_dom();
    $html_dom->load($html);
    
    // Format tables
    $tables = $html_dom->find('table');
    foreach($tables as $table){
      // if no table with is set expand to maximum size
      $table_width = isset($table->attr['width']) 
        ? $table->attr['width'] : 620;
      // make shure that the table is not bigger than the page
      if($table_width > 620) $table_width = 620;
      
      // get first row of the table
      $first_row = $table->find('tr', 0);
      if($first_row){

        // Get the columns: Look for <th> first and if not set look for <td>
        $cols = $first_row->find('th');
        if(!$cols) $cols = $first_row->find('td');
        
        if($cols){
          // get all columns and look if width is set
          $cols_width = 0;
          $remaining_cols = count($cols);
          foreach($cols as $col){
            if(isset($col->attr['width'])){
              $cols_width += $col->attr['width'];
              $remaining_cols--;
            }
          }
          // make shure that remaining cells is not 0
          if ($remaining_cols > 0) {
            // Calculate average cell space for the remaining cells
            $remaining_col_size = ($table_width - $cols_width) / $remaining_cols;
            // Set the column size for each element if no size is set
            foreach ($cols as $col) {
              if(!isset($col->attr['width'])){
                $col->attr['width'] = $remaining_col_size;
              }  
            }
          }
        }
      }
    }
    return $html_dom;
  } 

  /**
   * Replaces the link to image file that are private with stream link
   * Sets a relative filepath from the public folder for public images
   */
  private function prepare_images($html){
        
    $regexp = "<img[^>]+>";
    $images = array();
    if(preg_match_all("/$regexp/i", $html, $matches)) {
      $images = $matches[0];
    }
    
    foreach($images as $img_tag)
    {
      preg_match_all('/(src)=["\'](.*)["\']/i',$img_tag, $path_matches);
      $urls = $path_matches[2];
      foreach ($urls as $url) {
        //now get the file ID of the given file if it is a private file.
        $priv_path = '/system/files/';
        $private_file_dir_pos = strpos($url, $priv_path);
        if ($private_file_dir_pos !== false) {
          $end_pos = strlen($url) - ($private_file_dir_pos + strlen($priv_path));
          $file = substr($url, $private_file_dir_pos + strlen($priv_path), $end_pos); 

          $new_url = $GLOBALS['base_url'].'/'.variable_get('file_private_path').'/'.$file;
          $html = str_replace($url, $new_url, $html);
         
        }    
      }    
    }
    return $html;
  }
}  


function document_ooxml_export_get_settings(){
  
}

function document_ooxml_export_require_htmltodocx(){
  $path = libraries_get_path('htmltodocx');
  if(
    file_exists($path . '/phpword/PHPWord.php')
    && file_exists($path . '/simplehtmldom/simple_html_dom.php')
    && file_exists($path . '/htmltodocx_converter/h2d_htmlconverter.php')
  ){
    require_once($path . '/phpword/PHPWord.php');
    require_once($path . '/simplehtmldom/simple_html_dom.php');
    require_once($path . '/htmltodocx_converter/h2d_htmlconverter.php');
    return true;
  } else {
    drupal_set_message('The htmltodocx libraries are not installed correctly. Please download and extract to your libraries folder');
    return false;
  }
}


function document_ooxml_export_get_style(){

  $styles['default'] = 
    array (
      'size' => 11,
      'name' => 'Calibri',
    );
  
  $styles['elements'] = 
    array (
      'h1' => array (
        'bold' => TRUE,
        'size' => 20,
        ),
      'h2' => array (
        'bold' => TRUE,
        'size' => 16,
        'spaceAfter' => 150,
        ),
      'h3' => array (
        'size' => 14,
        'bold' => TRUE,
        'spaceAfter' => 100,
        ),
      'h4' => array (
        'size' => 12,
        'bold' => TRUE,
        'spaceAfter' => 100,
        ),
      'h5' => array (
        'size' => 12,
        'bold' => TRUE,
        'spaceAfter' => 50,
        ),
      'h6' => array (
        'size' => 12,
        'bold' => FALSE,
        'spaceAfter' => 50,
        ),
      'p' => array (
        'spaceBefore' => 15,
        'spaceAfter' => 45,
        ),
      'li' => array (
        ),
      'ol' => array (
        'spaceBefore' => 200,
        ),
      'ul' => array (
        'spaceAfter' => 150,
        ),
      'b' => array (
        'bold' => TRUE,
        ),
      'em' => array (
        'italic' => TRUE,
        ),
      'i' => array (
        'italic' => TRUE,
        ),
      'strong' => array (
        'bold' => TRUE,
        ),
      'b' => array (
        'bold' => TRUE,
        ),
      'code' => array (
        'name' => 'Courier New',
        ),
      'sup' => array (
        'superScript' => TRUE,
        'size' => 6,
        ), // Superscript not working in PHPWord 
      'u' => array (
        'underline' => PHPWord_Style_Font::UNDERLINE_SINGLE,
        ),
      'a' => array (
        'color' => '0000FF',
        'underline' => PHPWord_Style_Font::UNDERLINE_SINGLE,
        ),
      'table' => array (
        'borderColor' => '000000',  
        'borderSize' => 2,
        ),
      'th' => array (
        'borderColor' => '000000',
        'borderSize' => 2,
        'bold' => TRUE,
        ),
      'td' => array (
        'borderColor' => '000000',
        'borderSize' => 2,
        ),
      );
      
  $styles['classes'] = 
    array (
      'underline' => array (
        'underline' => PHPWord_Style_Font::UNDERLINE_SINGLE,
      ),
      'row-heading' => array (
        'bold' => TRUE,
      ),
      'table-of-contents' => array (
        'tabLeader' => 'underscore',
      ),
      'medium-image-size' => array (
        'width' => 150,
        'height' => 150,
      ),
      'small-image-size' => array (
        'width' => 80,
        'height' => 80,
      ),
    );
  
  $styles['inline'] = 
    array(
      'text-decoration: underline' => array (
        'underline' => PHPWord_Style_Font::UNDERLINE_SINGLE,
      ),
      'float: left' => array (
        'align' => 'left',
      ),
      'float: none' => array (
        'align' => 'center',
      ),
      'float: right' => array (
        'align' => 'right',
      ),
    );
    
  return $styles;
}
